@@ -139,6 +139,7 @@ pub enum TokenType {
     SymNomem,
     SymNoreturn,
     SymNostack,
+    SymNull,
     SymOptions,
     SymOut,
     SymPreservesFlags,
@@ -273,6 +274,7 @@ impl TokenType {
             SymNomem,
             SymNoreturn,
             SymNostack,
+            SymNull,
             SymOptions,
             SymOut,
             SymPreservesFlags,
@@ -348,6 +350,7 @@ impl TokenType {
             TokenType::SymNomem => Some(sym::nomem),
             TokenType::SymNoreturn => Some(sym::noreturn),
             TokenType::SymNostack => Some(sym::nostack),
+            TokenType::SymNull => Some(sym::null),
             TokenType::SymOptions => Some(sym::options),
             TokenType::SymOut => Some(sym::out),
             TokenType::SymPreservesFlags => Some(sym::preserves_flags),
@@ -562,6 +565,7 @@ macro_rules! exp {
     (Nomem)          => { exp!(@sym, nomem,           SymNomem) };
     (Noreturn)       => { exp!(@sym, noreturn,        SymNoreturn) };
     (Nostack)        => { exp!(@sym, nostack,         SymNostack) };
+    (Null)           => { exp!(@sym, null,            SymNull) };
     (Options)        => { exp!(@sym, options,         SymOptions) };
     (Out)            => { exp!(@sym, out,             SymOut) };
     (PreservesFlags) => { exp!(@sym, preserves_flags, SymPreservesFlags) };
