@@ -640,10 +640,10 @@ fn fn_abi_adjust_for_abi<'tcx>(
                 // we can't deduce any parameters for, so make sure the argument index is in
                 // bounds.
                 if let Some(deduced_param_attrs) = deduced_param_attrs.get(arg_idx)
-                    && deduced_param_attrs.read_only
+                    && deduced_param_attrs.read_only(tcx, cx.typing_env, arg.layout.ty)
                 {
-                    attrs.regular.insert(ArgAttribute::ReadOnly);
                     debug!("added deduced read-only attribute");
+                    attrs.regular.insert(ArgAttribute::ReadOnly);
                 }
             }
         }
